

 package org.jboss.tutorial.stateful.bean;



import javax.annotation.Resource;
import javax.ejb.*;
import javax.jms.*;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.io.Serializable;
import java.util.List;
import java.util.Properties;

@Stateless
@Local(ISyncLocalFacade.class)
@Remote(ISyncFacade.class)
public class SyncFacade implements ISyncFacade, Serializable
{



    /** For serialization purposes */
    private static final long serialVersionUID = 3392566934963731123L;

    public static final int NUM_RETRIES = 3;


    private static Queue responseSyncQueue;


    private static boolean isInitialized = false;

    private static final Object LOCK = new Object();

    @Resource(mappedName = "java:/jms/HornetQ/XACF")
    private static QueueConnectionFactory connectionFactory;

    /** Initializer for the queues */
    private static boolean init()
    {
        if (connectionFactory == null)
        {
            System.out.println("The ConnectionFactory is not available.");
        }

        synchronized (LOCK)
        {
            if (!isInitialized)
            {
                //Close and release any opened resourece
                clean();

                Properties p = new Properties();
                InitialContext iniCtx = null;

                String providerURL = "jnp://192.168.8.241:2501,192.168.8.51:2601";



                p.put("java.naming.factory.initial", "org.jnp.interfaces.NamingContextFactory");
                p.put("java.naming.provider.url", providerURL);
                p.put("java.naming.factory.url.pkgs", "org.jboss.naming:org.jnp.interfaces");


                try
                {
                    iniCtx = new InitialContext(p);


                    responseSyncQueue = (Queue) iniCtx.lookup("queue/ResponseSyncQueue");


                    isInitialized = true;
                }
                catch (NamingException e)
                {
                    isInitialized = false;
                    //clean();

                    System.out.println("Name not found. Details: " + e.getMessage());

                }
                finally
                {
                    try
                    {
                        if (iniCtx != null)
                        {
                            iniCtx.close();
                        }
                    }
                    catch (NamingException e)
                    {
                        System.out.println("Initial Context could not be closed# Details: " + e.getMessage());
                    }
                }
            }
            return true;
        }
    }


    private static void clean()
    {

        synchronized (LOCK)
        {
            isInitialized = false;
        }
    }





    public void addWO()
    {
        init();


        Session session = null;
        MessageProducer messageProducer = null;


        QueueConnection queueConnection = null;
        try
        {
            queueConnection = getQueueConnection();
            queueConnection.start();

            session = queueConnection.createSession(true, Session.AUTO_ACKNOWLEDGE);

            for (int i=0;i<10;i++)
            {

                //Start monitoring


                //retrieve workorder's destination queue

                messageProducer = session.createProducer(responseSyncQueue);

                System.out.println("Sending message no :"+i);

                ObjectMessage message = session.createObjectMessage("This is message nr "+i);

                //now push the message to queue
                messageProducer.send(message);
                messageProducer.close();

                System.out.println("Sent message no :"+i);


            }
        }
        catch (JMSException e)
        {
            //isInitialized = false;

            clean();

            System.out.println(
                   "JMS Exception occurred in SyncFacade. Rethrowing as EJBException."+
                    e);


        }
        finally
        {

            if (messageProducer != null)
            {
                try
                {
                    messageProducer.close();
                    messageProducer = null;
                }
                catch (JMSException e)
                {
                    System.out.println("Exception while closing producer " + e);
                }
            }
            if (session != null)
            {
                try
                {
                    session.close();
                    session = null;
                }
                catch (JMSException e)
                {
                    System.out.println("Exception while closing session " + e);
                }
            }

            if (queueConnection != null)
            {
                try
                {
                    queueConnection.close();
                    queueConnection = null;
                }
                catch (JMSException e)
                {
                    System.out.println("Exception while closing queue connection " + e);
                }
            }
        }
    }





    /**
     * Returns a queue connection
     *
     * @return - the Queue Connection
     * @throws JMSException - if any error occurred
     */
    public static QueueConnection getQueueConnection() throws JMSException
    {
        return connectionFactory.createQueueConnection();
    }

}

